import { strict as assert } from 'node:assert';

import { parse } from 'yaml';

const _meta = { draftSAdmonition: true };

function isDraft(input) {
  try {
    return parse(input).draft === true;
  } catch {
    return false;
  }
}

export function draftSAdmonition({
  text = '',
  type = '_meta',
  title = 'Drafted',
  version = 2,
} = {}) {
  assert(typeof title === 'string', new TypeError('`title` should be string'));
  assert(typeof type === 'string', new TypeError('`type` should be string'));
  assert(typeof text === 'string', new TypeError('`text` should be string'));
  assert([2, 3].includes(version), new TypeError('`version` should be 2 or 3'));

  return (tree) => {
    if (
      text &&
      tree.children.some(
        (node) => node.type === 'yaml' && isDraft(node.value),
      ) &&
      !tree.children.some((node) => node._meta?.draftSAdmonition)
    ) {
      const index =
        tree.children.findIndex(
          (node) => node.type === 'heading' && node.depth === 1,
        ) || 0;

      const context = {
        type: 'paragraph',
        children: [{ type: 'text', value: text }],
      };

      if (version === 2) {
        tree.children.splice(
          index + 1,
          0,
          {
            type: 'jsx',
            _meta,
            value: '<div hidden={!frontMatter.draft}>',
          },
          {
            _meta,
            type: 'admonitionHTML',
            data: {
              hName: 'admonition',
              hProperties: { title, type },
            },
            children: [context],
          },
          {
            type: 'jsx',
            _meta,
            value: '</div>',
          },
        );
      } else {
        tree.children.splice(index + 1, 0, {
          type: 'mdxJsxFlowElement',
          name: 'div',
          _meta,
          attributes: [
            {
              type: 'mdxJsxAttribute',
              name: 'hidden',
              value: {
                type: 'mdxJsxAttributeValueExpression',
                value: '!frontMatter.draft',
                data: {
                  estree: {
                    type: 'Program',
                    body: [
                      {
                        type: 'ExpressionStatement',
                        expression: {
                          type: 'UnaryExpression',
                          operator: '!',
                          prefix: true,
                          argument: {
                            type: 'MemberExpression',
                            object: {
                              type: 'Identifier',
                              name: 'frontMatter',
                            },
                            property: {
                              type: 'Identifier',
                              name: 'draft',
                            },
                            computed: false,
                            optional: false,
                          },
                        },
                      },
                    ],
                    sourceType: 'module',
                  },
                },
              },
            },
          ],
          children: [
            {
              type: 'containerDirective',
              name: type,
              children: [context],
            },
          ],
        });
      }
    }
  };
}
